@@ -1,6 +1,9 @@
 use std::{future::Future, marker::PhantomData, sync::Arc};
 
-use super::{After, AndThen, Around, Before, CatchError, InspectError, Map, MapToResponse};
+use super::{
+    After, AndThen, Around, Before, CatchAllError, CatchError, InspectAllError, InspectError, Map,
+    MapToResponse,
+};
 use crate::{
     error::IntoResult,
     middleware::{AddData, AddDataEndpoint},
@@ -15,6 +18,33 @@ pub trait Endpoint: Send + Sync {
 
     /// Get the response to the request.
     async fn call(&self, req: Request) -> Result<Self::Output>;
+
+    /// Get the response to the request and return a [`Response`].
+    ///
+    /// Unlike [`Endpoint::call`], when an error occurs, it will also convert
+    /// the error into a response object.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use poem::{error::NotFoundError, handler, http::StatusCode, Endpoint, Request, Result};
+    ///
+    /// #[handler]
+    /// fn index() -> Result<()> {
+    ///     Err(NotFoundError.into())
+    /// }
+    ///
+    /// # tokio::runtime::Runtime::new().unwrap().block_on(async {
+    /// let resp = index.get_response(Request::default()).await;
+    /// assert_eq!(resp.status(), StatusCode::NOT_FOUND);
+    /// # });
+    /// ```
+    async fn get_response(&self, req: Request) -> Response {
+        self.call(req)
+            .await
+            .map(IntoResponse::into_response)
+            .unwrap_or_else(|err| err.as_response())
+    }
 }
 
 struct SyncFnEndpoint<T, F> {
@@ -416,7 +446,7 @@ pub trait EndpointExt: IntoEndpoint {
     /// };
     ///
     /// let ep1 = make(|_| async { "hello" }).map_to_response();
-    /// let ep2 = make(|_| async { Err::<(), Error>(Error::new_with_status(StatusCode::BAD_REQUEST)) })
+    /// let ep2 = make(|_| async { Err::<(), Error>(Error::from_status(StatusCode::BAD_REQUEST)) })
     ///     .map_to_response();
     ///
     /// # tokio::runtime::Runtime::new().unwrap().block_on(async {
@@ -474,7 +504,7 @@ pub trait EndpointExt: IntoEndpoint {
     ///
     /// let ep1 = make(|_| async { "hello" })
     ///     .and_then(|value| async move { Ok(format!("{}, world!", value)) });
-    /// let ep2 = make(|_| async { Err::<String, _>(Error::new_with_status(StatusCode::BAD_REQUEST)) })
+    /// let ep2 = make(|_| async { Err::<String, _>(Error::from_status(StatusCode::BAD_REQUEST)) })
     ///     .and_then(|value| async move { Ok(format!("{}, world!", value)) });
     ///
     /// # tokio::runtime::Runtime::new().unwrap().block_on(async {
@@ -497,6 +527,58 @@ pub trait EndpointExt: IntoEndpoint {
         AndThen::new(self.into_endpoint(), f)
     }
 
+    /// Catch the all errors and convert it into a response.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use http::Uri;
+    /// use poem::{
+    ///     handler, http::StatusCode, web::Json, Endpoint, EndpointExt, Error, IntoResponse, Request,
+    ///     Response, Route,
+    /// };
+    /// use serde::Serialize;
+    ///
+    /// #[handler]
+    /// async fn index() {}
+    ///
+    /// let app = Route::new()
+    ///     .at("/index", index)
+    ///     .catch_all_error(custom_error);
+    ///
+    /// #[derive(Serialize)]
+    /// struct ErrorResponse {
+    ///     message: String,
+    /// }
+    ///
+    /// async fn custom_error(err: Error) -> impl IntoResponse {
+    ///     Json(ErrorResponse {
+    ///         message: err.to_string(),
+    ///     })
+    /// }
+    ///
+    /// # tokio::runtime::Runtime::new().unwrap().block_on(async {
+    /// let resp = app
+    ///     .call(Request::builder().uri(Uri::from_static("/abc")).finish())
+    ///     .await
+    ///     .unwrap();
+    /// assert_eq!(resp.status(), StatusCode::OK);
+    /// assert_eq!(
+    ///     resp.into_body().into_string().await.unwrap(),
+    ///     "{\"message\":\"not found\"}"
+    /// );
+    /// # })
+    /// ```
+    fn catch_all_error<F, Fut, R>(self, f: F) -> CatchAllError<Self, F, R>
+    where
+        F: Fn(Error) -> Fut + Send + Sync,
+        Fut: Future<Output = R> + Send,
+        R: IntoResponse + Send,
+        Self: Sized + Sync,
+    {
+        CatchAllError::new(self, f)
+    }
+
     /// Catch the specified type of error and convert it into a response.
     ///
     /// # Example
@@ -504,8 +586,8 @@ pub trait EndpointExt: IntoEndpoint {
     /// ```
     /// use http::Uri;
     /// use poem::{
-    ///     error::NotFoundError, handler, http::StatusCode, Endpoint, EndpointExt, Request, Response,
-    ///     Route,
+    ///     error::NotFoundError, handler, http::StatusCode, Endpoint, EndpointExt, IntoResponse,
+    ///     Request, Response, Route,
     /// };
     ///
     /// #[handler]
@@ -513,13 +595,12 @@ pub trait EndpointExt: IntoEndpoint {
     ///
     /// let app = Route::new().at("/index", index).catch_error(custom_404);
     ///
-    /// async fn custom_404(_: NotFoundError) -> Response {
-    ///     Response::builder()
-    ///         .status(StatusCode::NOT_FOUND)
-    ///         .body("custom not found")
+    /// async fn custom_404(_: NotFoundError) -> impl IntoResponse {
+    ///     "custom not found".with_status(StatusCode::NOT_FOUND)
     /// }
     ///
     /// # tokio::runtime::Runtime::new().unwrap().block_on(async {
+    ///
     /// let resp = app
     ///     .call(Request::builder().uri(Uri::from_static("/abc")).finish())
     ///     .await
@@ -531,10 +612,11 @@ pub trait EndpointExt: IntoEndpoint {
     /// );
     /// # })
     /// ```
-    fn catch_error<F, Fut, ErrType>(self, f: F) -> CatchError<Self, F, ErrType>
+    fn catch_error<F, Fut, R, ErrType>(self, f: F) -> CatchError<Self, F, R, ErrType>
     where
         F: Fn(ErrType) -> Fut + Send + Sync,
-        Fut: Future<Output = Response> + Send,
+        Fut: Future<Output = R> + Send,
+        R: IntoResponse + Send + Sync,
         ErrType: std::error::Error + Send + Sync + 'static,
         Self: Sized,
     {
@@ -551,14 +633,39 @@ pub trait EndpointExt: IntoEndpoint {
     /// #[handler]
     /// fn index() {}
     ///
-    /// let app = Route::new().at("/", index).inspect_err(|err| {
+    /// let app = Route::new().at("/", index).inspect_all_err(|err| {
     ///     println!("error: {}", err);
     /// });
     /// ```
-    fn inspect_err<F>(self, f: F) -> InspectError<Self, F>
+    fn inspect_all_err<F>(self, f: F) -> InspectAllError<Self, F>
     where
         F: Fn(&Error) + Send + Sync,
         Self: Sized,
+    {
+        InspectAllError::new(self, f)
+    }
+
+    /// Does something with each specified error type.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use poem::{error::NotFoundError, handler, EndpointExt, Route};
+    ///
+    /// #[handler]
+    /// fn index() {}
+    ///
+    /// let app = Route::new()
+    ///     .at("/", index)
+    ///     .inspect_err(|err: &NotFoundError| {
+    ///         println!("error: {}", err);
+    ///     });
+    /// ```
+    fn inspect_err<F, ErrType>(self, f: F) -> InspectError<Self, F, ErrType>
+    where
+        F: Fn(&ErrType) + Send + Sync,
+        ErrType: std::error::Error + Send + Sync + 'static,
+        Self: Sized,
     {
         InspectError::new(self, f)
     }
@@ -661,7 +768,7 @@ mod test {
             "abcdef"
         );
 
-        let err = make_sync(|_| Err::<String, _>(Error::new_with_status(StatusCode::BAD_REQUEST)))
+        let err = make_sync(|_| Err::<String, _>(Error::from_status(StatusCode::BAD_REQUEST)))
             .and_then(|resp| async move { Ok(resp + "def") })
             .call(Request::default())
             .await
@@ -680,7 +787,7 @@ mod test {
             "abcdef"
         );
 
-        let err = make_sync(|_| Err::<String, _>(Error::new_with_status(StatusCode::BAD_REQUEST)))
+        let err = make_sync(|_| Err::<String, _>(Error::from_status(StatusCode::BAD_REQUEST)))
             .map(|resp| async move { resp.to_string() + "def" })
             .call(Request::default())
             .await
